import os
import shutil
from ._download_helper import download_info, download_file
from tempfile import TemporaryDirectory
from os import path
import zipfile

home_url = "https://www.gyan.dev/ffmpeg/builds"


def get_version():
    return download_info(
        f"{home_url}/ffmpeg-release-essentials.zip.ver",
        "text/plain",
    )


def download_n_install(install_dir, progress=None, build_type=None):

    build_types = ("essentials", "full", "full-shared")
    if build_type is None:
        build_type = build_types[0]
    elif build_type not in build_types:
        raise ValueError(f"Invalid build_type specified. Must be one of {build_types}")

    with TemporaryDirectory() as tmpdir:
        zippath = path.join(tmpdir, "ffmpeg_win32.zip")
        url = f"{home_url}/ffmpeg-release-{build_type}.zip"
        download_file(zippath, url, "application/zip", progress=progress)

        with zipfile.ZipFile(zippath, "r") as f:
            f.extractall(tmpdir)

        _, (src_dir_name, *_), _ = next(os.walk(tmpdir))
        src_dir_path = path.join(tmpdir, src_dir_name)

        with open(path.join(src_dir_path, "VERSION"), "wt") as f:
            f.write(get_version())

        try:
            shutil.rmtree(install_dir)
        except:
            pass

        shutil.move(src_dir_path, install_dir)


def get_bindir(install_dir):
    return path.join(install_dir, "bin")



__version__ = "0.1.3"
__all__ = [
    "ffmpeg_dir",
    "ffmpeg_version",
    "ffmpeg_path",
    "ffprobe_path",
]

from os import listdir, path, rmdir, name as os_name
from shutil import rmtree
import sys
from appdirs import user_data_dir


if os_name == "nt":
    from ._win32 import (
        download_n_install,
        get_version as get_latest_version,
        get_bindir,
        home_url,
    )
elif sys.platform == "darwin":
    from ._macos import (
        download_n_install,
        get_version as get_latest_version,
        get_bindir,
        home_url,
    )
else:
    from ._linux import (
        download_n_install,
        get_version as get_latest_version,
        get_bindir,
        home_url,
    )

disclaimer_text = f"""
You are about to download the latest FFmpeg release build from {home_url}. 
Proceeding to download the file is done at your own discretion and risk and 
with agreement that you will be solely responsible for any damage to your 
computer system or loss of data that results from such activities. 

Do you wish to proceed to download? [yN] """

donation_text = f"""
Start downloading...

Please remember that to maintain and host the FFmpeg binaries is not free. 
If you appreciate their effort, please consider donating to help them with 
the upkeep of their website via {home_url}.
"""


def get_dir():
    return user_data_dir("ffmpeg_downloader", "ffmpegio")


def _ffmpeg_dir():
    return get_bindir(get_dir())


def _ffmpeg_version():
    try:
        with open(path.join(get_dir(), "VERSION"), "rt") as f:
            return f.read()
    except:
        return None


def __getattr__(name):  # per PEP 562
    try:
        return {
            "ffmpeg_dir": _ffmpeg_dir,
            "ffmpeg_path": lambda: _ffmpeg_version()
            and path.join(_ffmpeg_dir(), "ffmpeg" if os_name != "nt" else "ffmpeg.exe"),
            "ffprobe_path": lambda: _ffmpeg_version()
            and path.join(
                _ffmpeg_dir(), "ffprobe" if os_name != "nt" else "ffprobe.exe"
            ),
            "ffmpeg_version": _ffmpeg_version,
        }[name]()
    except:
        raise AttributeError(f"module '{__name__}' has no attribute '{name}'")


def has_update():
    current = __getattr__("ffmpeg_version")
    latest = get_latest_version()
    return current != latest


def update(skip_disclaimer=False, progress=None, **options):
    if has_update():

        if not skip_disclaimer:
            ans = input(disclaimer_text)
            if ans.lower() not in ("y", "yes"):
                print("\ndownload canceled")
                return
            print(donation_text)
        download_n_install(get_dir(), progress, **options)
        return False
    return True


def remove(ignore_errors=True):
    dir = get_dir()
    rmtree(dir, ignore_errors=ignore_errors)
    dir = path.dirname(dir)
    if not len(listdir(dir)):
        rmdir(dir)



from tempfile import TemporaryDirectory
from ._download_helper import download_info, download_file, chmod
import re, ssl, tarfile, os, shutil
from os import path

home_url = "https://johnvansickle.com/ffmpeg"


def get_version():

    return re.search(
        r"version: (\d+\.\d+(?:\.\d+)?)",
        download_info(f"{home_url}/release-readme.txt", "text/plain", 10),
    )[1]


def download_n_install(install_dir, progress=None, arch=None):
    archs = ("amd64", "i686", "arm64", "armhf", "armel")
    if arch is None:
        arch = archs[0]
    elif arch not in archs:
        raise ValueError(f"Invalid arch specified. Must be one of {arch}")

    with TemporaryDirectory() as tmpdir:
        tarpath = path.join(tmpdir, "ffmpeg_linux.tar.xz")
        url = f"{home_url}/releases/ffmpeg-release-{arch}-static.tar.xz"

        with TemporaryDirectory() as tmpdir:
            download_file(
                tarpath, url, "application/x-xz", progress=progress, timeout=10
            )

        with tarfile.open(tarpath, "r") as f:
            f.extractall(tmpdir)

        _, (src_dir_name, *_), _ = next(os.walk(tmpdir))
        src_dir_path = path.join(tmpdir, src_dir_name)

        with open(path.join(src_dir_path, "VERSION"), "wt") as f:
            f.write(get_version())

        try:
            shutil.rmtree(install_dir)
        except:
            pass

        # os.makedirs(install_dir, exist_ok=True)
        shutil.move(src_dir_path, install_dir)

    for cmd in ("ffmpeg", "ffprobe"):
        chmod(path.join(install_dir, cmd))


def get_bindir(install_dir):
    return install_dir



from urllib import request
from contextlib import contextmanager
import os, stat, ssl

ctx = ssl.create_default_context()
ctx.check_hostname = False
ctx.verify_mode = ssl.CERT_NONE


@contextmanager
def download_base(url, content_type, timeout=None):
    with request.urlopen(url, timeout=timeout or 5.0, context=ctx) as response:
        # pprint(response.headers.get_content_type())
        if response.headers.get_content_type() != content_type:
            raise RuntimeError(f'"{url}" is not the expected content type.')
        try:
            nbytes = int(response.getheader("content-length"))
        except:
            nbytes = 0
        yield response, nbytes


def download_info(url, content_type, timeout=None):
    with download_base(url, content_type, timeout) as (response, nbytes):
        info = response.read().decode("utf-8")
    return info


def download_file(outfile, url, content_type, progress=None, timeout=None):

    with download_base(url, content_type, timeout) as (response, nbytes):
        if progress:
            progress(0, nbytes)

        blksz = nbytes // 32 or 1024 * 1024
        with open(outfile, "wb") as f:
            nread = 0
            while True:
                b = response.read(blksz)
                if not b:
                    break
                f.write(b)
                nread += len(b)
                if progress:
                    progress(nread, nbytes)

    return nbytes


def chmod(binfile):
    # Set bits for execution and read for all users.
    exe_bits = stat.S_IXOTH | stat.S_IXUSR | stat.S_IXGRP
    read_bits = stat.S_IRUSR | stat.S_IRGRP | stat.S_IXGRP
    write_bits = stat.S_IWUSR | stat.S_IWGRP | stat.S_IWGRP
    os.chmod(binfile, exe_bits | read_bits | write_bits)



import argparse
import ffmpeg_downloader as ffdl
import tqdm

parser = argparse.ArgumentParser(description="Download latest FFmpeg release build")
parser.add_argument(
    "--update", "-U", "-u", action="store_true", help="Update to the latest release"
)
parser.add_argument(
    "--remove",
    "-r",
    "-R",
    "--delete",
    "-d",
    "-D",
    action="store_true",
    help="Remove downloaded FFmpeg build",
)

args = parser.parse_args()

ver = ffdl.ffmpeg_version

if args.remove:
    if ver is None:
        print("\nNo FFmpeg build has been downloaded.")
    elif input(f"\nAre you sure to remove the FFmpeg build v{ver}? [yN]: ").lower() in (
        "y",
        "yes",
    ):
        try:
            ffdl.remove(ignore_errors=False)
            print("\nFFmpeg build successfully removed.")
        except:
            print(
                f"\nFailed to delete FFmpeg build in {ffdl.ffmpeg_dir}. Please manually remove the directory."
            )
    else:
        print("\nFFmpeg build removal canceled.")
elif ver is None or args.update or ffdl.has_update():
    prog_data = {}
    try:

        def progress(nread, nbytes):
            if not len(prog_data):
                prog_data["tqdm"] = tqdm.tqdm(
                    desc="downloading", total=nbytes, unit=" bytes", leave=False
                )
                prog_data["last"] = 0
            pbar = prog_data["tqdm"]
            pbar.update(nread - prog_data["last"])
            prog_data["last"] = nread

        if ffdl.update(progress=progress):
            print("\nLatest version")
        else:
            print(f"FFmpeg v{ffdl.ffmpeg_version} successfully installed in {ffdl.ffmpeg_dir}\n")
    finally:
        pbar = prog_data.get("tqdm", None)
        if pbar:
            pbar.close()
else:
    print(f"FFmpeg v{ffdl.ffmpeg_version} already installed")



from tempfile import TemporaryDirectory
from os import path
import json, os, zipfile, shutil
from ._download_helper import download_info, download_file, chmod, download_base

home_url = "https://evermeet.cx/ffmpeg"


def get_version():
    return json.loads(
        download_info(f"{home_url}/info/ffmpeg/release", "application/json")
    )["version"]


def download_n_install(install_dir, progress=None):

    ntotal = 0

    def get_nbytes(cmd):
        with download_base(f"{home_url}/getrelease/{cmd}/zip", "application/zip") as (
            _,
            nbytes,
        ):
            return nbytes

    nfiles = [get_nbytes(cmd) for cmd in ("ffmpeg", "ffprobe")]
    ntotal = sum(nfiles)
    n1 = nfiles[0]
    prog = (
        lambda nread, nbytes: progress((nread + n1) if nbytes != n1 else nread, ntotal)
        if progress is not None
        else None
    )

    with TemporaryDirectory() as tmpdir:

        for cmd in ("ffmpeg", "ffprobe"):
            zippath = path.join(tmpdir, f"{cmd}.zip")
            download_file(
                zippath,
                f"{home_url}/getrelease/{cmd}/zip",
                "application/zip",
                progress=prog,
            )

            with zipfile.ZipFile(zippath, "r") as f:
                f.extractall(tmpdir)

            dst_path = path.join(install_dir, cmd)
            try:
                os.remove(dst_path)
            except:
                pass
            os.makedirs(install_dir, exist_ok=True)
            shutil.move(path.join(tmpdir, cmd), dst_path)
            chmod(dst_path)

        with open(path.join(install_dir, "VERSION"), "wt") as f:
            f.write(get_version())


def get_bindir(install_dir):
    return install_dir
